package com.jarvis.dataCenter.service.impl.report;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.jarvis.common.redis.service.RedisService;
import com.jarvis.dataCenter.bean.BeanUtils;
import com.jarvis.dataCenter.constants.DbConstants;
import com.jarvis.dataCenter.entity.report.ReportDoInfo;
import com.jarvis.dataCenter.entity.warehouse.DeliveryOrderDetailEntity;
import com.jarvis.dataCenter.entity.warehouse.DeliveryOrderEntity;
import com.jarvis.dataCenter.entity.warehouse.WarehouseEntity;
import com.jarvis.dataCenter.mapper.report.ReportDoInfoMapper;
import com.jarvis.dataCenter.mapper.warehouse.DeliveryOrderDetailMapper;
import com.jarvis.dataCenter.mapper.warehouse.DeliveryOrderMapper;
import com.jarvis.dataCenter.service.report.ReportCsOrderInfoService;
import com.jarvis.dataCenter.service.report.ReportDoInfoService;
import com.jarvis.dataCenter.service.warehouse.DeliveryOrderDetailService;
import com.jarvis.dataCenter.service.warehouse.DeliveryOrderService;
import com.jarvis.dataCenter.service.warehouse.WarehouseService;
import com.jarvis.lib.common.mybatisplus.service.impl.BaseServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @author jinzhaopo
 * @date 2019-10-30 14:57
 */
@DS(DbConstants.REPORT)
@Service
@Slf4j
public class ReportDoInfoServiceImpl extends BaseServiceImpl<ReportDoInfoMapper, ReportDoInfo> implements ReportDoInfoService {


    @Autowired
    private DeliveryOrderService deliveryOrderService;

    @Autowired
    private DeliveryOrderDetailService deliveryOrderDetailService;

    @Autowired
    private RedisService redisService;

    @Autowired
    private DeliveryOrderDetailMapper deliveryOrderDetailMapper;


    public static boolean SYNC = false;

    public static String REDIS_KEY = "sync:report:dodetail:id:";


    @Autowired
    private WarehouseService warehouseService;

    @Override
    public void sync() {
        Long endId = deliveryOrderDetailMapper.maxId();
        Integer startId = (Integer) redisService.get(REDIS_KEY);
        if (startId == null) {
            startId = 0;
        }

        //  同步信息
        for (long i = startId.intValue(); i <= endId.intValue(); i++) {
            try {
                sync(i);
            } catch (Exception e) {
                e.printStackTrace();
            }
            redisService.getRedisTemplateObj().opsForValue().increment(REDIS_KEY, 1);
        }
    }

    @Override
    public ReportDoInfo sync(Long doDetailId) {
        ReportDoInfo reportDoInfo = new ReportDoInfo();

        DeliveryOrderDetailEntity deliveryOrderDetailEntity = deliveryOrderDetailService.getById(doDetailId);
        BeanUtils.build(reportDoInfo, deliveryOrderDetailEntity);

        if(deliveryOrderDetailEntity == null){
            return null;
        }

        DeliveryOrderEntity deliveryOrderEntity =
                deliveryOrderService.getOne(Wrappers.lambdaQuery(new DeliveryOrderEntity()).eq(DeliveryOrderEntity::getCode, deliveryOrderDetailEntity.getDoCode()));
        BeanUtils.build(reportDoInfo, deliveryOrderEntity);

        if (deliveryOrderDetailEntity.getSupplierWarehouseId() != null) {
            WarehouseEntity warehouseEntity = warehouseService.getById(deliveryOrderDetailEntity.getSupplierWarehouseId());
            BeanUtils.build(reportDoInfo, warehouseEntity);
        }
        save(reportDoInfo);
        log.info("save:reportDoInfo:{}", reportDoInfo);
        return reportDoInfo;
    }
}
