package com.jarvis.dataCenter.service.impl.report;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.jarvis.common.redis.service.RedisService;
import com.jarvis.dataCenter.bean.BeanUtils;
import com.jarvis.dataCenter.constants.DbConstants;
import com.jarvis.dataCenter.entity.prod.BusMerchandisemasterEntity;
import com.jarvis.dataCenter.entity.prod.BusProductbrandEntity;
import com.jarvis.dataCenter.entity.prod.BusProductcategoryEntity;
import com.jarvis.dataCenter.entity.prod.PoSupliersmasterEntity;
import com.jarvis.dataCenter.entity.report.ReportSkuInfo;
import com.jarvis.dataCenter.mapper.prod.BusMerchandisemasterMapper;
import com.jarvis.dataCenter.mapper.report.ReportSkuInfoMapper;
import com.jarvis.dataCenter.service.prod.BusMerchandisemasterService;
import com.jarvis.dataCenter.service.prod.BusProductbrandService;
import com.jarvis.dataCenter.service.prod.BusProductcategoryService;
import com.jarvis.dataCenter.service.prod.PoSupliersmasterService;
import com.jarvis.dataCenter.service.report.ReportSkuInfoService;
import com.jarvis.lib.common.mybatisplus.service.impl.BaseServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @author jinzhaopo
 * @date 2019-10-29 11:04
 */
@DS(DbConstants.REPORT)
@Service
@Slf4j
public class ReportSkuInfoServiceImpl extends BaseServiceImpl<ReportSkuInfoMapper, ReportSkuInfo> implements ReportSkuInfoService {

    @Autowired
    private BusMerchandisemasterMapper busMerchandisemasterMapper;

    @Autowired
    private BusMerchandisemasterService busMerchandisemasterService;

    @Autowired
    private BusProductbrandService busProductbrandService;

    @Autowired
    private BusProductcategoryService busProductcategoryService;

    @Autowired
    private PoSupliersmasterService poSupliersmasterService;

    @Autowired
    private RedisService redisService;


    public static boolean SYNC = false;

    public static String REDIS_KEY = "sync:report:sku:id:";


    @Override
    public void sync() {
        Long endId = busMerchandisemasterMapper.maxId();
        Integer startId = (Integer) redisService.get(REDIS_KEY);
        if (startId == null) {
            startId = 0;
        }

        //  同步信息
        for (long i = startId.intValue(); i <= endId.intValue(); i++) {
            try {
                sync(i);
            } catch (Exception e) {
                e.printStackTrace();
            }
            redisService.getRedisTemplateObj().opsForValue().increment(REDIS_KEY, 1);
        }

    }

    @Override
    public ReportSkuInfo sync(Long skuId) {
        ReportSkuInfo reportSkuInfo = new ReportSkuInfo();

        //获取 bus_merchandisemaster 信息
        BusMerchandisemasterEntity busMerchandisemasterEntity =
                busMerchandisemasterService.getOne(Wrappers.lambdaQuery(new BusMerchandisemasterEntity()).eq(BusMerchandisemasterEntity::getId, skuId));
        BeanUtils.build(reportSkuInfo, busMerchandisemasterEntity);

        //获取 bus_productbrand 信息
        if (busMerchandisemasterEntity == null || busMerchandisemasterEntity.getBrandId() == null) {
            return null;
        }
        BusProductbrandEntity busProductbrandEntity =
                busProductbrandService.getOne(Wrappers.lambdaQuery(new BusProductbrandEntity()).eq(BusProductbrandEntity::getId, busMerchandisemasterEntity.getBrandId()));
        BeanUtils.build(reportSkuInfo, busProductbrandEntity);

        //获取三级分类 bus_productcategory 信息
        BusProductcategoryEntity productcategoryService3 =
                busProductcategoryService.getOne(Wrappers.lambdaQuery(new BusProductcategoryEntity()).eq(BusProductcategoryEntity::getId, busMerchandisemasterEntity.getCategoryId()));

        BusProductcategoryEntity productcategoryService2 =
                busProductcategoryService.getOne(Wrappers.lambdaQuery(new BusProductcategoryEntity()).eq(BusProductcategoryEntity::getId, productcategoryService3.getParentId()));

        BusProductcategoryEntity productcategoryService1 =
                busProductcategoryService.getOne(Wrappers.lambdaQuery(new BusProductcategoryEntity()).eq(BusProductcategoryEntity::getId, productcategoryService2.getParentId()));
        BeanUtils.build(reportSkuInfo, productcategoryService3, 3);
        BeanUtils.build(reportSkuInfo, productcategoryService2, 2);
        BeanUtils.build(reportSkuInfo, productcategoryService1, 1);


        //获取 po_supliersmaster
        PoSupliersmasterEntity poSupliersmasterEntity =
                poSupliersmasterService.getOne(Wrappers.lambdaQuery(new PoSupliersmasterEntity()).eq(PoSupliersmasterEntity::getId, busMerchandisemasterEntity.getSupliersId()));
        BeanUtils.build(reportSkuInfo, poSupliersmasterEntity);

        save(reportSkuInfo);
        log.info("save:reportSkuInfo:{}", reportSkuInfo);
        return reportSkuInfo;
    }
}
