package com.jarvis.dataCenter.service.impl.report;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.jarvis.common.redis.service.RedisService;
import com.jarvis.dataCenter.bean.BeanUtils;
import com.jarvis.dataCenter.constants.DbConstants;
import com.jarvis.dataCenter.entity.prod.*;
import com.jarvis.dataCenter.entity.report.ReportSoPoInfo;
import com.jarvis.dataCenter.mapper.prod.SoOrderdetailMapper;
import com.jarvis.dataCenter.mapper.report.ReportSoPoInfoMapper;
import com.jarvis.dataCenter.service.prod.*;
import com.jarvis.dataCenter.service.report.ReportSoPoInfoService;
import com.jarvis.lib.common.base.message.Message;
import com.jarvis.lib.common.mybatisplus.service.impl.BaseServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @author jinzhaopo
 * @date 2019-10-29 11:04
 */
@DS(DbConstants.REPORT)
@Service
@Slf4j
public class ReportSoPoInfoServiceImpl extends BaseServiceImpl<ReportSoPoInfoMapper, ReportSoPoInfo> implements ReportSoPoInfoService {

    @Autowired
    private PoOrdermasterService poOrdermasterService;

    @Autowired
    private SoOrderdetailService soOrderdetailService;

    @Autowired
    private SoOrdermasterService soOrdermasterService;

    @Autowired
    private RedisService redisService;

    @Autowired
    private PoCoupondetailService poCoupondetailService;

    @Autowired
    private PoCouponmasterService poCouponmasterService;

    @Autowired
    private SoOrderdetailMapper soOrderdetailMapper;

    public static String REDIS_KEY = "sync:report:sodetail:id:";


    @Override
    public void sync() {
        Long endId = soOrderdetailMapper.maxId();
        Integer startId = (Integer) redisService.get(REDIS_KEY);
        if (startId == null) {
            startId = 0;
        }

        //  同步信息
        for (long i = startId.intValue(); i <= endId.intValue(); i++) {
            try {
                sync(i);
            } catch (Exception e) {
                e.printStackTrace();
            }
            redisService.getRedisTemplateObj().opsForValue().increment(REDIS_KEY, 1);
        }
    }

    @Override
    public ReportSoPoInfo sync(Long soDetailId) {
        ReportSoPoInfo reportSoPoInfo = new ReportSoPoInfo();

        //获取要查询的 so_detail 信息
        SoOrderdetailEntity soOrderDetailEntity =
                soOrderdetailService.getOne(Wrappers.lambdaQuery(new SoOrderdetailEntity()).eq(SoOrderdetailEntity::getId, soDetailId));
        BeanUtils.build(reportSoPoInfo, soOrderDetailEntity);


        //获取 so 信息
        if (soOrderDetailEntity == null || StringUtils.isEmpty(soOrderDetailEntity.getOrderMasterCode())) {
            return null;
        }
        SoOrdermasterEntity soOrdermasterEntity =
                soOrdermasterService.getOne(Wrappers.lambdaQuery(new SoOrdermasterEntity()).eq(SoOrdermasterEntity::getOrderMasterCode, soOrderDetailEntity.getOrderMasterCode()));
        BeanUtils.build(reportSoPoInfo, soOrdermasterEntity);


        //获取 po 信息
        if (soOrdermasterEntity == null || StringUtils.isEmpty(soOrdermasterEntity.getPoOrderMasterCode())) {
            return null;
        }
        PoOrdermasterEntity poOrdermasterEntity =
                poOrdermasterService.getOne(Wrappers.lambdaQuery(new PoOrdermasterEntity()).eq(PoOrdermasterEntity::getOrderMasterCode, soOrdermasterEntity.getPoOrderMasterCode()));
        BeanUtils.build(reportSoPoInfo, poOrdermasterEntity);


        //-----------上面的是肯定会有的数据，下面是可能会有的数据


        //获取 coupon_detail 信息
        PoCoupondetailEntity poCoupondetailEntity =
                poCoupondetailService.getOne(Wrappers.lambdaQuery(new PoCoupondetailEntity()).eq(PoCoupondetailEntity::getOrderMasterCode, soOrderDetailEntity.getOrderMasterCode()));
        BeanUtils.build(reportSoPoInfo, poCoupondetailEntity);

        //获取 coupon 信息
        if (poCoupondetailEntity != null && poCoupondetailEntity.getCounponid() != null) {

            PoCouponmasterEntity poCouponmasterEntity =
                    poCouponmasterService.getOne(Wrappers.lambdaQuery(new PoCouponmasterEntity()).eq(PoCouponmasterEntity::getId, poCoupondetailEntity.getCounponid()));
            BeanUtils.build(reportSoPoInfo, poCouponmasterEntity);
        }

        save(reportSoPoInfo);
        log.info("save:reportSoPoInfo:{}", reportSoPoInfo);
        return reportSoPoInfo;
    }
}
